//
// Created by philipp on 29.12.17.
//

#ifndef NETWORKINGLIB_DATAGRAMSENDER_H
#define NETWORKINGLIB_DATAGRAMSENDER_H

#include "Utils.h"
#include "Networking.h"
#include "Message.h"

namespace networking
{
namespace message
{

template<typename Message>
class DatagramSender
{
public:
    using Socket = typename boost::asio::ip::udp::socket;
    using Endpoint = typename boost::asio::ip::udp::endpoint;

    DatagramSender(Networking & net)
        : net(net)
          , socket(net.getIoService())
    {
        socket.open(boost::asio::ip::udp::v4());
    }

    DatagramSender(const DatagramSender &) = delete;

    DatagramSender & operator=(const DatagramSender &) = delete;

    DatagramSender(DatagramSender &&) = delete;

    DatagramSender & operator=(DatagramSender &&) = delete;

    void send(const Message & message, const std::string & host, uint16_t port)
    {
        Endpoint endpoint{boost::asio::ip::address::from_string(host), port};
        sendDatagram(socket, message, endpoint);
    }

private:
    Networking & net;
    Socket socket;
};

}
}

#endif //NETWORKINGLIB_DATAGRAMSENDER_H
