//
// Created by philipp on 01.12.17.
//

#ifndef PROTOCOL_MESSAGES_H
#define PROTOCOL_MESSAGES_H

#include <cstdint>
#include <vector>
#include <boost/system/error_code.hpp>
#include <boost/asio/write.hpp>
#include <boost/asio/read.hpp>
#include "Utils.h"
#include "Networking.h"

namespace networking
{
namespace message
{

template<typename Message, typename SyncWriteStream>
void send(SyncWriteStream & socket, const Message & message)
{
    boost::asio::write(socket, boost::asio::buffer(message.data()));
};

template<typename Message, typename SyncWriteStream>
void send(Networking & net,
          SyncWriteStream & socket,
          const Message & message,
          const time::Duration & timeout)
{
    using namespace std::chrono_literals;
    if (timeout <= 0s)
        throw std::runtime_error("message::send() given timeout <= 0");

    networking::utils::write(net, socket, boost::asio::buffer(message.data()), timeout);
};

template<typename Message, typename SyncReadStream>
Message receive(SyncReadStream & socket)
{
    auto buf = Message::emptyBuffer();
    boost::asio::read(socket, boost::asio::buffer(buf));
    return Message(buf);
};

template<typename Message, typename SyncReadStream>
Message receive(Networking & net,
                SyncReadStream & socket,
                const time::Duration & timeout)
{
    using namespace std::chrono_literals;
    if (timeout <= 0s)
        throw std::runtime_error("message::receive() given timeout <= 0");

    auto buf = Message::emptyBuffer();
    networking::utils::read(net, socket, boost::asio::buffer(buf), timeout);
    return Message(buf);
};

template<typename Message, typename DatagramSocket, typename Endpoint>
void sendDatagram(DatagramSocket & socket, const Message & message, const Endpoint & endpoint)
{
    auto buf = message.data();
    socket.send_to(boost::asio::buffer(buf), endpoint);
}

}
}


#endif //PROTOCOL_MESSAGES_H
