//
// Created by philipp on 01.12.17.
//

#ifndef PROTOCOL_TCPNETWORKSERVICECLIENT_H
#define PROTOCOL_TCPNETWORKSERVICECLIENT_H

#include <string>
#include <functional>
#include <boost/asio/io_service.hpp>
#include <boost/asio/ip/tcp.hpp>
#include <boost/asio/connect.hpp>
#include "Message.h"
#include "Networking.h"

namespace networking
{
namespace service
{

template<typename Service>
class Client
{
public:
    using RequestMessage = typename Service::RequestMessage;
    using ResponseMessage = typename Service::ResponseMessage;

    using Endpoint = typename boost::asio::ip::tcp::endpoint;
    using Resolver = typename boost::asio::ip::tcp::resolver;
    using Socket = typename boost::asio::ip::tcp::socket;

    Client(Networking & net) : net(net)
    {}

    ResponseMessage call(const std::string & host,
                         uint16_t port,
                         const RequestMessage & request)
    {
        Resolver resolver(net.getIoService());
        auto serverEndpointIterator = resolver.resolve({host, std::to_string(port)});
        Socket socket(net.getIoService());
        boost::asio::connect(socket, serverEndpointIterator);
        message::send(socket, request);
        return message::receive<ResponseMessage>(socket);
    }

    ResponseMessage call(const std::string & host,
                         uint16_t port,
                         const RequestMessage & request,
                         time::Duration timeout)
    {
        // We have three places to lose time: connecting, sending and receiving.
        // So after each operation we subtract the time spend from our timeout.
        auto startTime = time::now();
        // Resolve endpoint.
        Resolver resolver(net.getIoService());
        auto serverEndpointIterator = resolver.resolve({host, std::to_string(port)});
        Socket socket(net.getIoService());
        // Connect to server.
        networking::utils::connect(net, socket, serverEndpointIterator, timeout);
        auto nowTime = time::now();
        auto timeSpend = nowTime - startTime;
        startTime = nowTime;
        timeout -= timeSpend;
        // Send the request.
        message::send(net, socket, request, timeout);
        nowTime = time::now();
        timeSpend = nowTime - startTime;
        timeout -= timeSpend;
        // Receive the response.
        return message::receive<ResponseMessage>(net, socket, timeout);
    }

private:
    networking::Networking & net;
};

}
}


#endif //PROTOCOL_TCPNETWORKSERVICECLIENT_H
