//
// Created by philipp on 28.12.17.
//

#ifndef NETWORKINGLIB_TIMER_H
#define NETWORKINGLIB_TIMER_H

#include "Networking.h"
#include <boost/asio.hpp>
#include <boost/asio/steady_timer.hpp>
#include "Time.h"

namespace networking
{
namespace time
{

class Timer : public std::enable_shared_from_this<Timer>
{
private:
    struct PrivateTag
    {
    };

public:
    using Ptr = std::shared_ptr<Timer>;
    using TimeoutHandler = std::function<void()>;

    // Objects of this class should always be declared as std::shared_ptr.
    Timer(PrivateTag, Networking & net);

    static Ptr create(Networking & net);

    Timer(const Timer &) = delete;

    Timer & operator=(const Timer &) = delete;

    Timer(Timer &&) = delete;

    Timer & operator=(Timer &&) = delete;

    void startTimeout(const time::Duration & duration, TimeoutHandler handler);

    void startPeriodicTimeout(const time::Duration & interval, TimeoutHandler handler);

    void disable();

    void enable();

    bool isEnabled();

private:
    boost::asio::basic_waitable_timer<time::Clock> timer;
    std::atomic<bool> enabled{true};

    void nextPeriod(const time::Duration & interval, TimeoutHandler handler);
};

}
}

#endif //NETWORKINGLIB_TIMER_H
