//
// Created by philipp on 28.12.17.
//

#include "../include/Timer.h"

using namespace networking;
using namespace time;

Timer::Timer(PrivateTag, Networking & net)
    : timer(net.getIoService())
{}

Timer::Ptr Timer::create(Networking & net)
{
    return std::make_shared<Timer>(PrivateTag{}, net);
}

void Timer::startTimeout(const time::Duration & duration, TimeoutHandler handler)
{
    if (!enabled)
        return;

    auto self = shared_from_this();
    timer.expires_from_now(duration);
    timer.async_wait(
        [self, handler](const boost::system::error_code & error)
        {
            if (error || !self->enabled)
                return;

            handler();
        });
}

void Timer::startPeriodicTimeout(const time::Duration & interval, Timer::TimeoutHandler handler)
{
    if (!enabled)
        return;

    auto self = shared_from_this();
    timer.expires_from_now(interval);
    timer.async_wait(
        [self, interval, handler](const boost::system::error_code & error)
        {
            if (error || !self->enabled)
                return;

            handler();
            self->nextPeriod(interval, handler);
        });
}

void Timer::disable()
{
    enabled = false;
    timer.cancel();
}

void Timer::enable()
{
    enabled = true;
}

bool Timer::isEnabled()
{
    return enabled;
}

void Timer::nextPeriod(const time::Duration & interval, Timer::TimeoutHandler handler)
{
    if (!enabled)
        return;

    auto self = shared_from_this();
    timer.expires_at(timer.expires_at() + interval);
    timer.async_wait(
        [self, interval, handler](const boost::system::error_code & error)
        {
            if (error || !self->enabled)
                return;

            handler();
            self->nextPeriod(interval, handler);
        });
}