//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_LEADERVEHICLE_H
#define PROTOCOL_LEADERVEHICLE_H


#include <mutex>
#include <unordered_map>
#include <bits/unordered_map.h>
#include "Vehicle.h"
#include "PlatoonService.h"
#include <NetworkingLib/DatagramSender.h>
#include <NetworkingLib/DatagramReceiver.h>

namespace protocol
{

class LeaderVehicle
    : public Vehicle, public std::enable_shared_from_this<LeaderVehicle>
{
private:
    struct PrivateTag
    {
    };

public:
    using Ptr = std::shared_ptr<LeaderVehicle>;

    static Ptr create(networking::Networking & net, VehicleId vehicleId);

    // This should not be used outside.
    LeaderVehicle(PrivateTag, networking::Networking & net, VehicleId vehicleId);

    std::size_t getNumFollowers();

    void setPlatoonConfig(const PlatoonConfig & platoonConfig);

protected:
    void doOnCreatePlatoon() override;

    void doCreatePlatoon() override;

    void doOnRunningPlatoon() override;

    void doRunningPlatoon() override;

    void doOnLeavingPlatoon() override;

    void doLeavingPlatoon() override;

private:
    static constexpr networking::time::Duration HEARTBEAT_TIMEOUT = std::chrono::milliseconds(1000);
    static constexpr networking::time::Duration BROADCAST_INTERVAL = std::chrono::milliseconds(10);

    struct Follower
    {
        VehicleEndpoint endpoint;
        networking::message::DatagramReceiver<PlatoonMessage>::Ptr heartbeatReceiver;
        networking::time::TimePoint timestamp;
    };

    std::unordered_map<VehicleId, Follower> followers{2};
    std::mutex followersMutex;
    std::vector<VehicleEndpoint>::iterator vehicleEndpointIter;

    networking::time::Timer::Ptr broadcastTimer;
    networking::message::DatagramSender<PlatoonMessage> broadcastSender;

    void makePlatoonCreateRequest(VehicleEndpoint vehicleEndpoint);

    PlatoonMessage makePlatoonCreateResponse(
        const tcp::endpoint & clientEndpoint, const PlatoonMessage & request);

    bool findFollower(VehicleId id);

    void addFollower(const VehicleEndpoint & endpoint);

    void removeFollower(Follower & follower);

    void receiveHeartbeats(Follower & follower);

    void startBroadcasting();

    void stopBroadcasting();
};

}


#endif //PROTOCOL_LEADERVEHICLE_H
