//
// Created by philipp on 30.11.17.
//

#ifndef PROTOCOL_VEHICLE_H
#define PROTOCOL_VEHICLE_H

#include <thread>
#include "VehicleEndpoint.h"
#include "PlatoonService.h"
#include <NetworkingLib/Networking.h>
#include <NetworkingLib/ServiceServer.h>
#include <NetworkingLib/ServiceClient.h>

namespace protocol
{
class Vehicle
{
public:
    using Ptr = std::shared_ptr<Vehicle>;

    using PlatoonCreateServiceServer = networking::service::Server<PlatoonService>;
    using PlatoonCreateServiceClient = networking::service::Client<PlatoonService>;

    enum class Role
    {
        LEADER, FOLLOWER
    };

    virtual ~Vehicle()
    {};

    bool run();

    bool isIdle();

    bool isPlatoonRunning();

    bool createPlatoon();

    bool leavePlatoon();

    Role getRole() const
    { return role; }

protected:
    Vehicle(
        networking::Networking & net,
        VehicleId vehicleId,
        Role role);

    enum class State
    {
        IDLE,
        ON_CREATING_PLATOON,
        CREATING_PLATOON,
        ON_RUNNING_PLATOON,
        RUNNING_PLATOON,
        ON_LEAVING_PLATOON,
        LEAVING_PLATOON
    };

    VehicleId vehicleId;
    PlatoonId platoonId;
    PlatoonConfig platoonConfig;
    std::mutex platoonConfigMutex;
    std::atomic<State> state{State::IDLE};
    networking::Networking & net;
    PlatoonCreateServiceServer::Ptr platoonCreateServiceServer;
    PlatoonCreateServiceClient platoonCreateServiceClient;
    std::vector<VehicleEndpoint> availableVehicleEndpoints;

    virtual void doOnCreatePlatoon() = 0;

    virtual void doCreatePlatoon() = 0;

    virtual void doOnRunningPlatoon() = 0;

    virtual void doRunningPlatoon() = 0;

    virtual void doOnLeavingPlatoon() = 0;

    virtual void doLeavingPlatoon() = 0;

    void scanAvailableVehicles();

private:
    Role role;
};
}


#endif //PROTOCOL_VEHICLE_H
