//
// Created by philipp on 30.11.17.
//

#include <vector>
#include <algorithm>
#include <iostream>
#include "../include/Vehicle.h"

using namespace protocol;

Vehicle::Vehicle(networking::Networking & net, VehicleId vehicleId, Vehicle::Role role)
    : net(net)
      , vehicleId(vehicleId)
      , role(role)
      , platoonCreateServiceClient(net)
{}

bool Vehicle::run()
{
    switch (state.load())
    {
        case State::ON_CREATING_PLATOON:
            doOnCreatePlatoon();
            return true;

        case State::CREATING_PLATOON:
            doCreatePlatoon();
            return true;

        case State::ON_RUNNING_PLATOON:
            doOnRunningPlatoon();
            return true;

        case State::RUNNING_PLATOON:
            doRunningPlatoon();
            return true;

        case State::ON_LEAVING_PLATOON:
            doOnLeavingPlatoon();
            return true;

        case State::LEAVING_PLATOON:
            doLeavingPlatoon();
            return true;
    }

    // State::IDLE
    return false;
}

bool Vehicle::isIdle()
{
    return state.load() == State::IDLE;
}

bool Vehicle::isPlatoonRunning()
{
    return state.load() == State::RUNNING_PLATOON;
}

bool Vehicle::createPlatoon()
{
    if (state.load() != State::IDLE)
        return false;

    state = State::ON_CREATING_PLATOON;
    return true;
}

bool Vehicle::leavePlatoon()
{
    if (state.load() != State::RUNNING_PLATOON)
        return false;

    state = State::ON_LEAVING_PLATOON;
    return true;
}

void Vehicle::scanAvailableVehicles()
{
    // TODO: change vehicle endpoint information!
    availableVehicleEndpoints = {
        VehicleEndpoint{1, "127.0.0.1", 10001, 10011},
        VehicleEndpoint{2, "127.0.0.1", 10002, 10022},
        VehicleEndpoint{3, "127.0.0.1", 10003, 10033},
    };

    // remove me out of the list (I cannot connect to myself)
    availableVehicleEndpoints.erase(std::remove_if(availableVehicleEndpoints.begin(), availableVehicleEndpoints.end(),
                                                   [this](VehicleEndpoint & endpoint) -> bool
                                                   {
                                                       return endpoint.getVehicleId() == vehicleId;
                                                   }), availableVehicleEndpoints.end());
}
